#ifndef GETC_C
#define GETC_C

/* Small c library, by Jeremy Dewey (jxd45@po.cwru.edu)
   http://www.seashellinc.com/jdewey

   Synopsis:
      getc()

   Description:
      Waits for the user to press a key, returns the ASCII value of the key
      translated (i.e. pressing '5' returns 'R'). Also supports use of the 2nd
      and ALPHA keys, when GETC_CMP is defined to allow input of lowercase and
      numeric characters.

   Example: 
      #define GETC_CMP
      ...
      ch = getc()

   Notes:

*/

extern char key_tran[0x39];
#ifdef GETC_CMP
extern char key_trn2[0x39];
extern char key_trn3[0x39];

getc() {
   char state, c;

   state = 0;
   while (1) {
      c = getkey();
      if (c == K_SECOND) {
         if (state == 2) state = 0;
         else state = 2;
      } else if (c == K_ALPHA) {
         if (state == 1) state = 0;
         else state = 1;
      }
      else {
         if (state == 0) return key_trn2[c];
         else if (state == 1) return key_tran[c];
         else return key_trn3[c];
      }
   }
}
#endif
#ifndef GETC_CMP
getc() {
   return key_tran[getkey()];
}
#endif

#asm
key_tran:
.db 0,1,2,3,4,0,0,0,0,9,
.db 'X','T','O','J','E',$0f,0
.db ' ','W','S','N','I','D',$17,0
.db 'Z','V','R','M','H','C',$1f,$20
.db 'Y','U','Q','L','G','B',$27,$28
.db $29,'=','P','K','F','A',$2e,$2f,$30
.db $31,$32,$33,$34,$35,$36,$37,$38
#endasm
#ifdef GETC_CMP
#asm
key_trn2:
.db 0,1,2,3,4,0,0,0,0,9,
.db 'x','t','o','j','e',$0f,0
.db ' ','w','s','n','i','d',$17,0
.db 'z','v','r','m','h','c',$1f,$20
.db 'y','u','q','l','g','b',$27,$28
.db $29,'=','p','k','f','a',$2e,$2f,$30
.db $31,$32,$33,$34,$35,$36,$37,$38
key_trn3:
.db 0,1,2,3,4,0,0,0,0,9,
.db '+','-','*','/','^',$0f,0
.db ' ','3','6','9','i','d',$17,0
.db '.','2','5','8','h','c',$1f,$20
.db '0','1','4','7','g','b',$27,$28
.db $29,':',',','k','f','a',$2e,$2f,$30
.db $31,$32,$33,$34,$35,$36,$37,$38
#endasm
#endif

#endif
