#ifndef LINE_C
#define LINE_C

/* Small c library, by Jeremy Dewey (jxd45@po.cwru.edu)
   http://www.seashellinc.com/jdewey

   Synopsis:
      line(char x1, char y1, char x2, char y2)

   Description:
      Draws a line from point (x1,y1) to point (x2,y2)

   Example: 
      line(0,0,10,10);

   Notes:
      You must call graphpage() before calling this routine

;       ------------------------------------------------------------
;          Optimized Implementation of Bresenham's line algorithm 
;         by Stephane JANTZEN (Stephane.Jantzen@scinfo.u-nancy.fr)
;
;                             Version 2 (01/31/96)
;       ------------------------------------------------------------
;
; This version replaces the previous one, which was a little bigger, and
; contained a bug for repeated lines. The little demo routine allow you to
; move the line with the arrows, just to see how fast it is redrawn. Just
; notice that redrawing time is principally spent clearing the screen.
;
; * parameters : 
;    - DE : coordinates of the 1st point
;    - HL : coordinates of the 2nd point
;
; * code size : 106 bytes
;
; The size could be smaller, but not without a significant loss of speed
; (using pushes and pops)
;
; Feel free to end all optimizations to the address above. I would especially
; like a solution to remove the "(way)" call, without losing speed of space.
; I would also be interested in a fast put_pixel routine.
;
; Many thanx to Pascal Bouron (bouron@Yoko.ENS-Cachan.fr) for his precious help

*/

char _lnway;

line (x1,y1,x2,y2) char x1,y1,x2,y2; {
#asm
   ; Grab arguments off stack

        ld hl,4
        add hl,sp
        ld e,(hl)

        ld hl,2
        add hl,sp
        ld d,(hl)

        ld hl,8
        add hl,sp
        ld c,(hl)

        ld hl,6
        add hl,sp
        ld h,(hl)
        ld l,c

        ld      A,D
        sub     H
        jr      NC,_lndxpos    ;delta_x > 0
_lndxneg:
        neg
_lndxpos:
        ld      B,A              ;B <- |delta_x|
        ld      A,E
        sub     L
        jr      NC,_lndypos    ;delta_y > 0
_lndyneg:
        neg
_lndypos:
        sub     B               ;|delta_y|
        push    AF
        jr      C,_lndx        ;|delta_x| > |delta_y|
_lndy:
        ld      A,H             ;if |delta_x| < |delta_y| then
        ld      H,L             ;then values x and y are swapped
        ld      L,A             ;so the loop will always be performed on the
        ld      A,D             ;x value. A flag must be set to
        ld      D,E             ;remind that data must be drawn (y,x)
        ld      E,A             ;instead of (x,y)

        ;ld      A,$01
        ;ld      (Flag),A        ;the Flag !!!
_lndx:
        ld      A,D
        sub     H
        jr      NC,_lntdy        ;x1 < x2
_lntdx:
        ex	DE,HL
_lntdy:
        ld      A,E
        sub     L
        ld      A,$01
        jr      NC,_lnsta
        neg                     ;y1 > y2 : in _lncase2 the 'y' variable
_lnsta:
        ld      (_lnway),A
_lninitl:
        ld      B,H
        ld      C,L
        ld      A,E
        sub     L
        jr      NC,_lneinit
        ld      A,L
        ld      L,E
        ld      E,A
_lneinit:
        ld      A,E
        sub     L
        rla
        ld      L,A             ;value to add in _lncase1 (d < 0)
        add     A,H
        sub     D
        ld      E,A             ;'d' variable is initialised
        add     A,H
        sub     D
        ld      H,A             ;value to add in _lncase2 (d >= 0)
_lnloop:
        ld      A,B
        sub     D
        jr      NC,_lnend       ;the line is completely drawn.
        pop     AF
        ;ld      A,(Flag)
        bit     7,A
        push    AF
        push    AF
        push    BC
        jr      Z,_lndpt
        ld      A,B
        ld      B,C
        ld      C,A
_lndpt:
        push de
        push hl
        CALL_(_putpix)
        pop  hl
        pop  de
        pop     BC
        pop     AF
;TestD:
        bit     7,E
        jr      NZ,_lncase1
_lncase2:                          ;d >= 0 
        ld      A,E
        add     A,H
        ld      E,A
        ld      A,(_lnway)
        add     A,C
        ld      C,A
        jr      _lneloop
_lncase1:                          ;d < 0
        ld      A,E
        add     A,L
        ld      E,A
_lneloop:
        inc     B
        jr      _lnloop
_lnend:
        pop     AF              ;MUST NOT BE REMOVED
#endasm
}

#endif